rm(list=ls())
library(gamlss)
### load("unif600_opSBC_edge.RData")

###------------------------------------
###--------- GAMLSS fitting on 500x12 sample 
###------------------------------------
## To obtain good starting values based on the GAMLSS model from the 500 x 12 pilot sample
#NOTE:  load data from 500 x 12 pilot sample  && fit model based on good ini values (i.e., run the lines until the model fitting)

load("equidist_df_opt_edge.RData")

GAMLSS<-gamlss((-Y) ~ cs(X, df = opSBC$par[1]), sigma.fo = ~cs(X, df = opSBC$par[2]), nu.fo = ~1, family=RGE, method=mixed(5,50), mu.start=mu.ini, sigma.start=sig.ini, nu.start=shp.ini, trace = TRUE)

X<-data.frame(X)


###------------------------------------
###--------- Data setup of Unif sample
###------------------------------------

# Randomly choose partial observations
set.seed(100)
n.choose <- 600
### chosen.ind <- sample(1:6000, n.choose)

#Build X
X.new <- read.csv("X_unif600.csv")
X.new <- X.new$x
#X.new  <- as.numeric(X.new)
#X.new  <- X.new [chosen.ind]

#Build Y
loadtype = 2; #1: flapwise, 2: edgewise
Y_jk <- read.csv("Y_edge_Unif600.csv")
Y_jk <-Y_jk$x
#Y <- Y_jk[,loadtype]
#Y <- Y[chosen.ind]
Y <- Y_jk


###------------------------------------
###--------- Find good initial parameters
###------------------------------------
X.new <- data.frame(X.new)
colnames(X.new)<-"X"


mu.new <- predict(GAMLSS,newdata=X.new, data=X,what="mu",type="response")  
sig.new <- predict(GAMLSS,newdata=X.new, data=X,what="sigma",type="response")  

#Good starting values
mu.ini  <- mu.new
sig.ini <- sig.new
shp.ini <- rep(GAMLSS$nu.fv[1], n.choose)  #negation of  'shape_param_edge'

#Replace the equi pilot X with the unif pilot x
X<- as.numeric(X.new[,1])

###------------------------------------
###--------- Optimize GAMLSS fitting on the small Unif sample in terms of BIC
###------------------------------------

#GAMLSS<-gamlss((-Y) ~ cs(X, df = opSBC$par[1]), sigma.fo = ~cs(X, df = opSBC$par[2]), nu.fo = ~1, family=RGE, method=mixed(5,50), mu.start=mu.ini, sigma.start=sig.ini, nu.start=shp.ini, trace = TRUE)

## Optimize SBC (k = log N;  k = 2 for AIC) to select the smoothing parameter (a function of 'df'): p. 22-4 of GAMLSS in R (2007) JSS
fn.SBC <- function(p) AIC(gamlss((-Y) ~ cs(X, df = p[1]), sigma.fo = ~cs(X, df = p[2]), nu.fo = ~1, family=RGE, method=mixed(5,50), mu.start=mu.ini, sigma.start=sig.ini, nu.start=shp.ini, trace = T), k = log(length(Y))) # k = 2 for AIC
upper.df <- min(20,length(unique(X)))  #you must supply 1 < df <= n,  n = #{unique x} = 12
opSBC <- optim(par = c(7.250863, 3.000000), fn.SBC, method = "L-BFGS-B", lower=c(2,2), upper = c(upper.df,upper.df), control = list(trace = 6, REPORT = 1))  #Output: 7.264465 2.895282



###------------------------------------
###--------- Prep to plot
###------------------------------------

##Fitting
GAMLSS<-gamlss((-Y) ~ cs(X, df = 7.264465), sigma.fo = ~cs(X, df = 2.895282), nu.fo = ~1, family=RGE, method=mixed(5,50), mu.start=mu.ini, sigma.start=sig.ini, nu.start=shp.ini, trace = TRUE)


## Predict mu at different x
X.new <- seq(3,25,by=0.01) #0.00001)  #0.01
X.new <- data.frame(X.new)
colnames(X.new)<-"X"
X<-data.frame(X)
## colnames(X)
mu.new <- predict(GAMLSS,newdata=X.new, data=X,what="mu",type="response")  

x11()
plot(X.new$X, -mu.new, type="l", xlab="x",ylab=expression(mu)) #plot(X.new$X, mu.new)
  #lines(X$X, fitted(GAMLSS,what="mu"),col="red")

write.csv(mu.new, file="mu.new_edge_uni600.csv", row.names = F)
#write.csv(mu.new, file="mu.new_edge_uni600_by00001.csv", row.names = F)


## Predict sigma at different x
sig.new <- predict(GAMLSS,newdata=X.new, data=X,what="sigma",type="response")  
x11()
plot(X.new$X, sig.new, type="l", xlab="x",ylab=expression(sigma))  #plot(X.new$X, sig.new)
  #lines(X$X, fitted(GAMLSS,what="sigma"),col="red")

write.csv(sig.new, file="sig.new_edge_uni600.csv", row.names = F)
#write.csv(sig.new, file="sig.new_edge_uni600_by00001.csv", row.names = F)

## Predict shape parameter
fitted(GAMLSS, what="nu")[1]



### --------------------------------------
### Test on Edge
### --------------------------------------

## Standardize
Z <- (-Y-GAMLSS$mu.fv)/GAMLSS$sigma.fv

## KS test
KS <- ks.test(Z, "pRGE", mu= 0, sigma= 1, nu = GAMLSS$nu.fv[1])
print(signif(KS$p.value,3))

## QQ plot
x11()
#Z.th <- rRGE(10^6, mu= 0, sigma= 1, nu = GAMLSS$nu.fv[1])
Z.th <- qRGE((rank(Z)-0.5)/length(Z),mu= 0, sigma= 1, nu = GAMLSS$nu.fv[1]) 
#Theoretical quantile:  http://onlinestatbook.com/2/advanced_graphs/q-q_plots.html
qqplot(as.numeric(Z), Z.th, xlab="Sample quantiles", ylab="Theoretical quantiles", xlim=c(-9,3), ylim=c(-9,3)) #xlab="Data", ylab="rRGE")
abline(a=0, b =1)
#qqline(as.numeric(Z), distribution=qRGE)#, xlab="Sample", ylab="rRGE") #xlab="Data", ylab="rRGE")

